"use strict";
/**
 * Copyright (c) 2020-2021 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.InterUnitBondLineVisual = exports.InterUnitBondLineParams = void 0;
var tslib_1 = require("tslib");
var param_definition_1 = require("../../../mol-util/param-definition");
var structure_1 = require("../../../mol-model/structure");
var linear_algebra_1 = require("../../../mol-math/linear-algebra");
var mol_util_1 = require("../../../mol-util");
var link_1 = require("./util/link");
var complex_visual_1 = require("../complex-visual");
var types_1 = require("../../../mol-model/structure/model/types");
var bond_1 = require("./util/bond");
var lines_1 = require("../../../mol-geo/geometry/lines/lines");
var geometry_1 = require("../../../mol-math/geometry");
var tmpRefPosBondIt = new structure_1.Bond.ElementBondIterator();
function setRefPosition(pos, structure, unit, index) {
    tmpRefPosBondIt.setElement(structure, unit, index);
    while (tmpRefPosBondIt.hasNext) {
        var bA = tmpRefPosBondIt.move();
        bA.otherUnit.conformation.position(bA.otherUnit.elements[bA.otherIndex], pos);
        return pos;
    }
    return null;
}
function createInterUnitBondLines(ctx, structure, theme, props, lines) {
    var bonds = structure.interUnitBonds;
    var edgeCount = bonds.edgeCount, edges = bonds.edges;
    var sizeFactor = props.sizeFactor, aromaticBonds = props.aromaticBonds;
    if (!edgeCount)
        return lines_1.Lines.createEmpty(lines);
    var ref = linear_algebra_1.Vec3();
    var loc = structure_1.StructureElement.Location.create();
    var builderProps = {
        linkCount: edgeCount,
        referencePosition: function (edgeIndex) {
            var b = edges[edgeIndex];
            var unitA, unitB;
            var indexA, indexB;
            if (b.unitA < b.unitB) {
                unitA = structure.unitMap.get(b.unitA);
                unitB = structure.unitMap.get(b.unitB);
                indexA = b.indexA;
                indexB = b.indexB;
            }
            else if (b.unitA > b.unitB) {
                unitA = structure.unitMap.get(b.unitB);
                unitB = structure.unitMap.get(b.unitA);
                indexA = b.indexB;
                indexB = b.indexA;
            }
            else {
                throw new Error('same units in createInterUnitBondLines');
            }
            return setRefPosition(ref, structure, unitA, indexA) || setRefPosition(ref, structure, unitB, indexB);
        },
        position: function (posA, posB, edgeIndex) {
            var b = edges[edgeIndex];
            var uA = structure.unitMap.get(b.unitA);
            var uB = structure.unitMap.get(b.unitB);
            uA.conformation.position(uA.elements[b.indexA], posA);
            uB.conformation.position(uB.elements[b.indexB], posB);
        },
        style: function (edgeIndex) {
            var o = edges[edgeIndex].props.order;
            var f = mol_util_1.BitFlags.create(edges[edgeIndex].props.flag);
            if (types_1.BondType.is(f, 2 /* MetallicCoordination */) || types_1.BondType.is(f, 4 /* HydrogenBond */)) {
                // show metallic coordinations and hydrogen bonds with dashed cylinders
                return 1 /* Dashed */;
            }
            else if (o === 3) {
                return 3 /* Triple */;
            }
            else if (aromaticBonds && types_1.BondType.is(f, 16 /* Aromatic */)) {
                return 5 /* Aromatic */;
            }
            else if (o === 2) {
                return 2 /* Double */;
            }
            return 0 /* Solid */;
        },
        radius: function (edgeIndex) {
            var b = edges[edgeIndex];
            loc.structure = structure;
            loc.unit = structure.unitMap.get(b.unitA);
            loc.element = loc.unit.elements[b.indexA];
            var sizeA = theme.size.size(loc);
            loc.unit = structure.unitMap.get(b.unitB);
            loc.element = loc.unit.elements[b.indexB];
            var sizeB = theme.size.size(loc);
            return Math.min(sizeA, sizeB) * sizeFactor;
        },
        ignore: bond_1.makeInterBondIgnoreTest(structure, props)
    };
    var l = link_1.createLinkLines(ctx, builderProps, props, lines);
    var child = structure.child;
    var sphere = geometry_1.Sphere3D.expand(geometry_1.Sphere3D(), (child !== null && child !== void 0 ? child : structure).boundary.sphere, 1 * props.sizeFactor);
    l.setBoundingSphere(sphere);
    return l;
}
exports.InterUnitBondLineParams = tslib_1.__assign(tslib_1.__assign(tslib_1.__assign({}, complex_visual_1.ComplexLinesParams), bond_1.BondLineParams), { includeParent: param_definition_1.ParamDefinition.Boolean(false) });
function InterUnitBondLineVisual(materialId) {
    return complex_visual_1.ComplexLinesVisual({
        defaultProps: param_definition_1.ParamDefinition.getDefaultValues(exports.InterUnitBondLineParams),
        createGeometry: createInterUnitBondLines,
        createLocationIterator: bond_1.BondIterator.fromStructure,
        getLoci: bond_1.getInterBondLoci,
        eachLocation: bond_1.eachInterBond,
        setUpdateState: function (state, newProps, currentProps, newTheme, currentTheme, newStructure, currentStructure) {
            state.createGeometry = (newProps.sizeFactor !== currentProps.sizeFactor ||
                newProps.linkScale !== currentProps.linkScale ||
                newProps.linkSpacing !== currentProps.linkSpacing ||
                newProps.dashCount !== currentProps.dashCount ||
                newProps.ignoreHydrogens !== currentProps.ignoreHydrogens ||
                !mol_util_1.arrayEqual(newProps.includeTypes, currentProps.includeTypes) ||
                !mol_util_1.arrayEqual(newProps.excludeTypes, currentProps.excludeTypes));
            if (newStructure.interUnitBonds !== currentStructure.interUnitBonds) {
                state.createGeometry = true;
                state.updateTransform = true;
                state.updateColor = true;
                state.updateSize = true;
            }
        }
    }, materialId);
}
exports.InterUnitBondLineVisual = InterUnitBondLineVisual;
//# sourceMappingURL=bond-inter-unit-line.js.map